/* file stevents.c	G. Moody        13 May 1999
			Last revised:	3 November 1999
-------------------------------------------------------------------------------
stevents: prepare ST annotations for plotting
Copyright (C) 1999 George B. Moody

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA.

You may contact the author by e-mail (george@mit.edu) or postal mail
(MIT Room E25-505A, Cambridge, MA 02139 USA).  For updates to this software,
please visit the author's web site (http://ecg.mit.edu/).
_______________________________________________________________________________
*/

#include <stdio.h>
#ifndef __STDC__
extern void exit();
#endif

#include <wfdb/wfdb.h>
#include <wfdb/ecgcodes.h>

/* values for annot_type */
#define INVALID 0
#define GREF	1
#define LREF	2
#define ISTS	3
#define ISTX	4
#define ISTE	5
#define RSTS	6
#define RSTX	7
#define RSTE	8
#define AXIS	9

char *pname;

main(argc, argv)
int argc;
char *argv[];
{
    char *record = NULL, *prog_name();
    double sph, spm, sps, atof(), fabs();
    static int i, st[3], yb, yt;
    static long episode_start[WFDB_MAXSIG], rrep_start[WFDB_MAXSIG],
      from = 0L, to = 0L, t;
    static WFDB_Anninfo ai;
    WFDB_Annotation annot;
    void help();
    int annot_type = INVALID;
#ifndef CUSTOM_BARS
    int ybase[] = { 10, -10 };	/* y-coords of beginning of bars for each
				   signal */
    int ysc[ ] = { 1, -1 };	/* y-coord multiplier for each signal (-1
				   means invert bars) */
    /* Default bar lengths (<0 means extend down from ybase) */
    int dy[] = { 0,	/* invalid */
 		 -8,	/* global reference */
		 -5,	/* local reference */
		 30,	/* start of ischemic ST episode */
		 32,	/* extremum of ischemic ST episode */
		 30,	/* end of ischemic ST episode */
		 20,	/* start of rate-related ST episode */
		 22,	/* extremum of rate-related ST episode */
		 20,	/* end of rate-related ST episode */
		 15	/* axis shift */
    };
    int dyir[2] = {0, 10}; /* bottom and top (relative to ybase) of filled-in
			      rectangle for ischemic ST episodes */
    int dyrr[2] = {0, 5}; /* bottom and top (relative to ybase) of filled-in
			      rectangle for rate-related ST episodes */
#else
#include "custom_bars.c"
#endif

    pname = prog_name(argv[0]);
    ai.name = "sta"; ai.stat = WFDB_READ;

    /* Interpret command-line options. */
    for (i = 1; i < argc; i++) {
	if (*argv[i] == '-') switch (*(argv[i]+1)) {
	  case 'a':	/* event annotator follows */
	    if (++i >= argc) {
		(void)fprintf(stderr, "%s: event annotator must follow -a\n",
			      pname);
		exit(1);
	    }
	    ai.name = argv[i];
	    break;
	  case 'f':	/* starting time follows */
	    if (++i >= argc) {
		(void)fprintf(stderr, "%s: starting time must follow -f\n",
			      pname);
		exit(1);
	    }
	    from = i;		/* to be converted to sample intervals below */
	    break;
	  case 'h':	/* print usage summary and quit */
	    help();
	    exit(0);
	    break;
	  case 'r':	/* input record name follows */
	    if (++i >= argc) {
		(void)fprintf(stderr,
			      "%s: input record name must follow -r\n",
			      pname);
		exit(1);
	    }
	    record = argv[i];
	    break;
	  case 't':	/* ending time follows */
	    if (++i >= argc) {
		(void)fprintf(stderr, "%s: end time must follow -t\n", pname);
		exit(1);
	    }
	    to = i;
	    break;
	  default:
	    (void)fprintf(stderr, "%s: unrecognized option %s\n",
			  pname, argv[i]);
	    exit(1);
	}
	else {
	    (void)fprintf(stderr, "%s: unrecognized argument %s\n",
			  pname, argv[i]);
	    exit(1);
	}
    }
    if (record == NULL) {
	help();
	exit(1);
    }

    if ((sps = sampfreq(record)) < 0.)
	(void)setsampfreq(sps = WFDB_DEFFREQ);
    spm = sps * 60.0;
    sph = spm * 60.0;

    if (annopen(record, &ai, 1) < 0) /* open annotation file */
	exit(2);

    if (from && iannsettime(strtim(argv[(int)from])) < 0) exit(2);
    if (to) to = strtim(argv[(int)to]);
    else to = strtim("e");	/* end of record */
    if (to <= 0L) to = strtim("24:0:0");

    while (getann(0, &annot) == 0 && (annot.time <= to)) {
	char *p, *q;

	if (annot.anntyp != STCH) continue;
	switch (*(annot.aux+1)) {
	case 'G': /* global reference */
	    annot_type = GREF; break;
	case 'L': /* local reference */
	    annot_type = LREF; break;
	case '(': /* start of episode */
	    if (*(annot.aux+2) == 's') {
	      episode_start[annot.chan] = annot.time;
	      annot_type = ISTS; /* start of ST episode */
	    }
	    else if (*(annot.aux+2) == 'r') {
	      rrep_start[annot.chan] = annot.time;
	      annot_type = RSTS; /* rate-related */
	    }
	    else annot_type = INVALID;
	    break;
	case 'a': /* extremum or axis shift */
	    if (*(annot.aux+2) == 'x')
		annot_type = AXIS; /* axis shift */
	    else if (*(annot.aux+2) == 's')
		annot_type = ISTX; /* ST episode */
	    else if (*(annot.aux+2) == 'r')
		annot_type = RSTX; /* rate-related ST */
	    else annot_type = INVALID;
	    break;
	case 'r': /* end of rate related episode */
	    annot_type = RSTE; break;
	case 's': /* end of ST episode */
	    annot_type = ISTE; break;
	default:
	    annot_type = INVALID; break;
	}

	if (annot.chan == 0 || annot.chan == 1) {
	    printf("%g %d %d %d\n", annot.time/sph, ybase[annot.chan],
		   ybase[annot.chan] + ysc[annot.chan]*dy[annot_type],
		   annot_type);
	    /* duplicate the first line to work around a PS bug */
	    if (annot_type == GREF)
		printf("%g %d %d\n", annot.time/sph, ybase[annot.chan],
		       ybase[annot.chan] + ysc[annot.chan]*dy[annot_type]);
	    if (annot_type == ISTE) {
		/* end of ischemic episode -- fill in a rectangle */
		if ((t = episode_start[annot.chan]) > 0L) {
		    yb = ybase[annot.chan] + ysc[annot.chan]*dyir[0];
		    yt = ybase[annot.chan] + ysc[annot.chan]*dyir[1];
		    while ((t += sps) < annot.time)
			printf("%g %d %d\n", t/sph, yb, yt);
		    episode_start[annot.chan] = -1L;
		}
	    }
	    if (annot_type == RSTE) {
		/* end of rate-related episode -- fill in a rectangle */
		if ((t = rrep_start[annot.chan]) > 0L) {
		    yb = ybase[annot.chan] + ysc[annot.chan]*dyrr[0];
		    yt = ybase[annot.chan] + ysc[annot.chan]*dyrr[1];
		    while ((t += sps) < annot.time)
			printf("%g %d %d\n", t/sph, yb, yt);
		    rrep_start[annot.chan] = -1L;
		}
	    }
	}
    }
    if ((t = episode_start[annot.chan]) > 0L) {
	yb = ybase[annot.chan] + ysc[annot.chan]*dyir[0];
	yt = ybase[annot.chan] + ysc[annot.chan]*dyir[1];
	while ((t += sps) < to)
	    printf("%g %d %d\n", t/sph, yb, yt);
    }
    else if ((t = rrep_start[annot.chan]) > 0L) {
	yb = ybase[annot.chan] + ysc[annot.chan]*dyrr[0];
	yt = ybase[annot.chan] + ysc[annot.chan]*dyrr[1];
	while ((t += sps) < to)
	    printf("%g %d %d\n", t/sph, yb, yt);
    }

    exit(0);			/*NOTREACHED*/
}

char *prog_name(s)
char *s;
{
    char *p = s + strlen(s);

#ifdef MSDOS
    while (p >= s && *p != '\\' && *p != ':') {
	if (*p == '.')
	    *p = '\0';			/* strip off extension */
	if ('A' <= *p && *p <= 'Z')
	    *p += 'a' - 'A';		/* convert to lower case */
	p--;
    }
#else
    while (p >= s && *p != '/')
	p--;
#endif
    return (p+1);
}

static char *help_strings[] = {
    "usage: %s -r RECORD [OPTIONS ...]\n",
    "where RECORD specifies the input, and OPTIONS may include:",
    " -a EVENT-ANNOTATOR         use EVENT-ANNOTATOR (default: sta)",
    " -f TIME                    start at specified TIME",
    " -h                         print this usage summary",
    " -t TIME                    stop at specified TIME",
    "Output is three columns: elapsed time (hours), y0, y1",
    NULL
};

void help()
{
    int i;

    (void)fprintf(stderr, help_strings[0], pname);
    for (i = 1; help_strings[i] != NULL; i++)
	(void)fprintf(stderr, "%s\n", help_strings[i]);
}
