/* file stevents.c	G. Moody        13 May 1999

-------------------------------------------------------------------------------
stevents: prepare ST annotations for plotting
Copyright (C) 1999 George B. Moody

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA.

You may contact the author by e-mail (george@mit.edu) or postal mail
(MIT Room E25-505A, Cambridge, MA 02139 USA).  For updates to this software,
please visit the author's web site (http://ecg.mit.edu/).
_______________________________________________________________________________

This program is based on ihr.c, from the WFDB Software Package.
*/

#include <stdio.h>
#ifndef __STDC__
extern void exit();
#endif

#include <wfdb/wfdb.h>
#define map2
#define ammap
#define mamap
#define annpos
#include <wfdb/ecgmap.h>

char *pname;

main(argc, argv)
int argc;
char *argv[];
{
    char *record = NULL, *prog_name();
    double ihr, ihrlast, mhr = 70., sph, spm, sps, tol = 10.0, atof(), fabs();
    static int i, j, st[3];
    long from = 0L, to = 0L, lasttime;
    static WFDB_Anninfo ai[2];
    WFDB_Annotation annot;
    void help();

    pname = prog_name(argv[0]);
    ai[0].name = "sta"; ai[0].stat = WFDB_READ;
    ai[1].name = "16a"; ai[1].stat = WFDB_READ;

    /* Interpret command-line options. */
    for (i = 1; i < argc; i++) {
	if (*argv[i] == '-') switch (*(argv[i]+1)) {
	  case 'a':	/* event annotator follows */
	    if (++i >= argc) {
		(void)fprintf(stderr, "%s: event annotator must follow -a\n",
			      pname);
		exit(1);
	    }
	    ai[0].name = argv[i];
	    break;
	  case 'A':	/* measurement annotator follows */
	    if (++i >= argc) {
		(void)fprintf(stderr,
			      "%s: measurement annotator must follow -a\n",
			      pname);
		exit(1);
	    }
	    ai[1].name = argv[i];
	    break;
	  case 'd':	/* tolerance (max HR change beat to beat, in bpm) */
	    if (++i >= argc) {
		(void)fprintf(stderr, "%s: tolerance must follow -d\n",
			      pname);
		exit(1);
	    }
	    tol = atof(argv[i]);
	    break;
	  case 'f':	/* starting time follows */
	    if (++i >= argc) {
		(void)fprintf(stderr, "%s: starting time must follow -f\n",
			      pname);
		exit(1);
	    }
	    from = i;		/* to be converted to sample intervals below */
	    break;
	  case 'h':	/* print usage summary and quit */
	    help();
	    exit(0);
	    break;
	  case 'r':	/* input record name follows */
	    if (++i >= argc) {
		(void)fprintf(stderr,
			      "%s: input record name must follow -r\n",
			      pname);
		exit(1);
	    }
	    record = argv[i];
	    break;
	  case 't':	/* ending time follows */
	    if (++i >= argc) {
		(void)fprintf(stderr, "%s: end time must follow -t\n", pname);
		exit(1);
	    }
	    to = i;
	    break;
	  default:
	    (void)fprintf(stderr, "%s: unrecognized option %s\n",
			  pname, argv[i]);
	    exit(1);
	}
	else {
	    (void)fprintf(stderr, "%s: unrecognized argument %s\n",
			  pname, argv[i]);
	    exit(1);
	}
    }
    if (record == NULL) {
	help();
	exit(1);
    }

    if ((sps = sampfreq(record)) < 0.)
	(void)setsampfreq(sps = WFDB_DEFFREQ);
    spm = sps * 60.0;
    sph = spm * 60.0;

    if (annopen(record, ai, 2) < 0) /* open annotation file */
	exit(2);

    if (from && iannsettime(strtim(argv[(int)from])) < 0) exit(2);
    if (to) to = strtim(argv[(int)to]);

    while (getann(0, &annot) == 0 && (to == 0L || annot.time <= to)) {
	char *p, *q;

	if (annot.anntyp != STCH) continue;
	switch (*(annot.aux+1)) {
	case 'G': /* global reference */
	    i = 10; break;
	case 'L': /* local reference */
	    i = 8; break;
	case '(': /* start of episode */
	    if (*(annot.aux+2) == 's') i = 30; /* start of ST episode */
	    else if (*(annot.aux+2) == 'r') i = 20; 	/* rate-related */
	    else i = 0;
	    break;
	case 'a': /* extremum or axis shift */
	    if (*(annot.aux+2) == 'x') i = 15;	/* axis shift */
	    else if (*(annot.aux+2) == 's') i = 32;	/* ST episode */
	    else if (*(annot.aux+2) == 'r') i = 22;	/* rate-related ST */
	    else i = 0;
	    break;
	case 'r': /* end of rate related episode */
	    i = 20; break;
	case 's': /* end of ST episode */
	    i = 30; break;
	default:
	    i = 0; break;
	}
	if (i == 10) {  /* global reference -- get absolute ST level */
	  WFDB_Annotation m;
	  while (getann(1, &m) == 0 &&
		 (m.time < annot.time || m.chan != annot.chan))
	    ;
	  if (m.aux) {
	    char *p, *q;
	    p = m.aux+4;
	    while (*p != '+' && *p != '-')
		p++;
	    q = p+1;
	    st[m.chan] = 0;
	    while ('0' <= *q && *q <= '9') {
		st[m.chan] *= 10;
		st[m.chan] += *q++ - '0';
	    }
	    if (*p == '-') st[m.chan] = -st[m.chan];
	  }
	}

	if (i == 8 || i == 10) {
	  printf("%ld", annot.time);
	    j = atoi(annot.aux+6) + st[annot.chan];
	    printf(" %d %d\n", annot.chan, j);
	}
    }
    exit(0);			/*NOTREACHED*/
}

char *prog_name(s)
char *s;
{
    char *p = s + strlen(s);

#ifdef MSDOS
    while (p >= s && *p != '\\' && *p != ':') {
	if (*p == '.')
	    *p = '\0';			/* strip off extension */
	if ('A' <= *p && *p <= 'Z')
	    *p += 'a' - 'A';		/* convert to lower case */
	p--;
    }
#else
    while (p >= s && *p != '/')
	p--;
#endif
    return (p+1);
}

static char *help_strings[] = {
    "usage: %s -r RECORD -a ANNOTATOR [OPTIONS ...]\n",
    "where RECORD and ANNOTATOR specify the input, and OPTIONS may include:",
    " -d TOL   reject beat-to-beat HR changes > TOL bpm",
    " -f TIME  start at specified TIME",
    " -h       print this usage summary",
    " -t TIME  stop at specified TIME",
    "Output is four columns: elapsed time (hours), HR (bpm), ST0, ST1",
    NULL
};

void help()
{
    int i;

    (void)fprintf(stderr, help_strings[0], pname);
    for (i = 1; help_strings[i] != NULL; i++)
	(void)fprintf(stderr, "%s\n", help_strings[i]);
}
